package edu.udo.cs.uschi;

import java.lang.Double;

public class MyBeta
{
    double alpha; // standard parameter of the beta distribution
    double beta;  // standard parameter of the beta distribution
    double p; // new parameter: p = a/(a+b)
    double N; // new parameter: N = a+b
    double B; // B(alpha,beta)
    double lnB; // log(B(alpha,beta))

    static long maxit = 100; // maximum newton iterations
    static double epsilon = 1e-8; // precision

    public MyBeta()
	throws Exception
    {
	setAlphaBeta(1,1);
    };


    public MyBeta(double a, double b)
	throws Exception
    {
	setAlphaBeta(a,b);
    };


    public void setAlphaBeta(double a,double b)
	throws Exception
    {
	alpha = a;
	beta = b;
	p = alpha / (alpha+beta);
	N = alpha+beta;
	if(! ((alpha > 0) && (beta > 0))){
	    throw(new Exception("Beta-parameters must be positive: "+alpha+", "+beta));
	};
	lnB = lngamma(alpha)+lngamma(beta)-lngamma(alpha+beta);
	B = Math.exp(lnB);
    };


    public void setPN(double new_p, double new_N)
	throws Exception
    {
	p = new_p;
	N = new_N;
	alpha = p*N;
	beta = N-alpha;
	if(! ((alpha > 0) && (beta > 0))){
	    throw(new Exception("Beta-parameters must be positive: "+alpha+", "+beta));
	};
	lnB = lngamma(alpha)+lngamma(beta)-lngamma(alpha+beta);
	B = Math.exp(lnB);
    };


    public double lngamma(double x)
    {
	double y;
	double tmp;
	double ser;
	y = x;
	tmp = x+5.5;
	tmp -= (x+0.5)*Math.log(tmp);
	ser = 1.000000000190015;
	ser += 76.18009172947146/(1.0+y);
	ser += -86.50532032941677/(2.0+y);
	ser += 24.01409824083091/(3.0+y);
	ser += -1.2317395724250155/(4.0+y);
	ser += 0.1208650973866179e-2/(5.0+y);
	ser += -0.5395239384953e-5/(6.0+y);
	return(-tmp+Math.log(2.5066282746310005*ser/x));
    };


    public double Mathabs(double x){
	if(x<0.0){
	    x = -x;
	};
	return x;
    };

    public double betacf(double a, double b, double x)
	throws Exception
    {
	long m;
	long m2;
	double aa;
	double c;
	double d;
	double del;
	double  h;
	double qab;
	double qam;
	double qap;
	qab = a+b;
	qap = a+1.0;
	qam = a-1.0;
	c = 1.0;
	d = 1.0-qab*x/qap;
	if(Mathabs(d) < 1e-30){
	    d = 1e30;
	}
	else{
	    d = 1.0/d;
	};
	h = d;
	del = 0;
	for(m=1;m<=100;m++){
	    m2 = 2*m;
	    aa=m*(b-m)*x/((qam+m2)*(a+m2));
	    d = 1.0+aa*d;
	    if(Mathabs(d) < 1e-30){
		d = 1e-30;
	    };
	    c = 1.0+aa/c;
	    if(Mathabs(c) < 1e-30){
		c = 1e-30;
	    };
	    d = 1.0/d;
	    h *= d*c;
	    aa = -(a+m)*(qab+m)*x/((a+m2)*(qap+m2));
	    d = 1.0+aa*d;
	    if(Mathabs(d) < 1e-30){
		d = 1e-30;
	    };
	    c = 1.0+aa/c;
	    if(Mathabs(c) < 1e-30){
		c = 1e-30;
	    };
	    d = 1.0/d;
	    del = d*c;
	    h *= del;
	    if(Mathabs(del-1.0) < 3e-7){
		m = 101;
	    };
	};
	if(Mathabs(del-1.0) >= 3e-7){
	    throw(new Exception("Convergence Error in betacf"));
	};
	return h;
    };


    public double distributionFunction(double x)
	throws Exception
    {
	double res;
	if((x < 0.0) || (x > 1.0)){
	    throw(new Exception("Error in Beta.distributionFunction: x must be in [0.1]"));
	};
	if(x == 0.0){
	    res = 0.0;
	}
	else if(x == 1.0){
	    res = 1.0;
	}
	else{
	    res = Math.exp(-lnB+alpha*Math.log(x)+beta*Math.log(1.0-x));
	    if(x < (alpha+1.0)/(alpha+beta+2.0)){
		res = res*betacf(alpha,beta,x)/alpha;
	    }
	    else{
		res = 1.0-res*betacf(beta,alpha,1.0-x)/beta;
	    };
	};
	return res;
    };


    /**
     * use Newton method to calculate inverse of beta distribution function
     */ 
    public double inverseDistributionFunction(double y)
	throws Exception
    {
	if(y<=0){ return 0; };
	if(y>=1){ return 1; };
	double x=0.5;
	double old_x=0;
	long it=0;
	double error = epsilon+1;
	while((error > epsilon) && (it<maxit) && (old_x != x)){
	    it++;
	    old_x = x;
	    x = x - (distributionFunction(x)-y) / (Math.pow(x,alpha-1)*Math.pow(1-x,beta-1))*B;
	    // keep x in ]0,1[
	    if(x<=0.0){
		x = old_x/2;
	    }
	    else if(x>=1.0){
		x = (1.0+old_x)/2;
	    };
	    error = y-distributionFunction(x);
	    if(error < 0){ error = -error; };

	};
	return x;
    };


    public static void main(String[] argv)
	throws Exception
    {
	// checks
	double x = Double.parseDouble(argv[0]);
	double a = Double.parseDouble(argv[1]);
	double b = Double.parseDouble(argv[2]);
	System.out.println("F = Beta("+a+","+b+"), x = "+x);
	MyBeta Beta = new MyBeta(a,b);
	System.out.println("F = "+Beta.distributionFunction(x));
	System.out.println("F^-1 = "+Beta.inverseDistributionFunction(x));
	System.out.println("FF^-1 = "+Beta.distributionFunction(Beta.inverseDistributionFunction(x)));
	System.out.println("F^-1F = "+Beta.inverseDistributionFunction(Beta.distributionFunction(x)));
    }
};
