package edu.udo.cs.uschi.scaler;

import edu.udo.cs.uschi.MembershipObject;
import edu.udo.cs.uschi.MembershipVector;
import edu.udo.cs.uschi.scaler.TrivialScaler;
import java.util.Properties;
import java.io.*;

public class MinmaxScaler extends TrivialScaler
{
    double min;


    public MinmaxScaler()
    {
    };


    public MinmaxScaler(Properties p)
    {
	parameters = p;
    };


    public void init(MembershipVector memberships)
    {
	min = Double.POSITIVE_INFINITY;
	double[] memb;
	int i = memberships.size();
	int j;
	while(i>0){
	    i--;
	    memb = memberships.membershipsAt(i);
	    j=memb.length;
	    while(j>0){
		j--;
		if(memb[j] < min){
		    min = memb[j];
		};
	    };
	};
    };


    public MembershipObject scale(MembershipObject unscaled)
    {
	double[] m = unscaled.memberships;
	double[] n = new double[m.length];
	double sum=0;
	for(int i=0;i<m.length;i++){
	    n[i] = (m[i]-min);
	    if(n[i] < 0){
		n[i] = 0;
	    };
	    sum+=n[i];
	};
	if(sum > 0.0){
	    for(int i=0;i<m.length;i++){
		n[i] /= sum;
	    };
	}
	else{
	    // identical confidence for every class
	    for(int i=0;i<m.length;i++){
		n[i] = 1/m.length;
	    };
	};
	MembershipObject scaled = new MembershipObject(unscaled.trueclass,n);
	return scaled;
    };


    public void read(BufferedReader infile)
	throws Exception
    {
	String line=null;
	boolean ok;
	ok = false;
	while(!ok){
	    line = infile.readLine();
	    ok = true;
	    if(line != null){
		line.trim();
		if(line.length() == 0){
		    ok = false;
		}
		else if(line.charAt(0) == '#'){
		    ok = false;
		};
	    };
	};
	if(line == null){
	    throw(new Exception("No minmax parameter found in file."));
	};
	// now we must be at the "min" line
	int pos = line.indexOf(' ');
	min = Double.parseDouble(line.substring(pos));
    };


    public void write(BufferedWriter outfile)
	throws Exception
    {
	outfile.write("# MinmaxScaler");
	outfile.newLine();
	outfile.write("min ");
	outfile.write(Double.toString(min));
	outfile.newLine();
    };
};
